@@ -293,7 +293,7 @@ class GridConfiguration(ConfigurationBase):
             global_key='image.max_shrink_factor')
 
         if conf.get('origin') is None:
-            log.warn('grid %s does not have an origin. default origin will change from sw (south/west) to nw (north-west) with MapProxy 2.0',
+            log.warning('grid %s does not have an origin. default origin will change from sw (south/west) to nw (north-west) with MapProxy 2.0',
                 conf['name'],
             )
 
@@ -318,6 +318,22 @@ class GridConfiguration(ConfigurationBase):
         return grid
 
 
+def preferred_srs(conf):
+    from mapproxy.srs import SRS, PreferredSrcSRS
+
+    preferred_conf = conf.get('preferred_src_proj', {})
+
+    if not preferred_conf:
+        return
+
+    preferred = PreferredSrcSRS()
+
+    for target, preferred_srcs in preferred_conf.items():
+        preferred.add(SRS(target), [SRS(s) for s in preferred_srcs])
+
+    return preferred
+
+
 class GlobalConfiguration(ConfigurationBase):
     def __init__(self, conf_base_dir, conf, context):
         ConfigurationBase.__init__(self, conf, context)
@@ -327,6 +343,7 @@ class GlobalConfiguration(ConfigurationBase):
         finish_base_config(self.base_config)
 
         self.image_options = ImageOptionsConfiguration(self.conf.get('image', {}), context)
+        self.preferred_srs = preferred_srs(self.conf.get('srs', {}))
         self.renderd_address = self.get_value('renderd.address')
 
     def _copy_conf_values(self, d, target):
@@ -395,6 +412,11 @@ class ImageOptionsConfiguration(ConfigurationBase):
         jpeg_quality = options.pop('jpeg_quality', None)
         if jpeg_quality and not isinstance(jpeg_quality, int):
             raise ConfigurationError('jpeg_quality is not an integer')
+
+        tiff_compression = options.pop('tiff_compression', None)
+        if tiff_compression and tiff_compression not in ('raw', 'tiff_lzw', 'jpeg'):
+            raise ConfigurationError('unknown tiff_compression')
+
         quantizer = options.pop('quantizer', None)
         if quantizer and quantizer not in ('fastoctree', 'mediancut'):
             raise ConfigurationError('unknown quantizer')
@@ -549,6 +571,14 @@ class SourceConfiguration(ConfigurationBase):
             self.conf.setdefault('image', {})['transparent'] = self.conf['transparent']
         return self.context.globals.image_options.image_opts(self.conf.get('image', {}), format)
 
+    def supported_srs(self):
+        from mapproxy.srs import SRS, SupportedSRS
+
+        supported_srs = [SRS(code) for code in self.conf.get('supported_srs', [])]
+        if not supported_srs:
+            return None
+        return SupportedSRS(supported_srs, self.context.globals.preferred_srs)
+
     def http_client(self, url):
         from mapproxy.client.http import auth_data_from_url, HTTPClient
 
@@ -561,10 +591,11 @@ class SourceConfiguration(ConfigurationBase):
 
         timeout = self.context.globals.get_value('http.client_timeout', self.conf)
         headers = self.context.globals.get_value('http.headers', self.conf)
+        hide_error_details = self.context.globals.get_value('http.hide_error_details', self.conf)
 
         http_client = HTTPClient(url, username, password, insecure=insecure,
                                  ssl_ca_certs=ssl_ca_certs, timeout=timeout,
-                                 headers=headers)
+                                 headers=headers, hide_error_details=hide_error_details)
         return http_client, url
 
     @memoize
@@ -614,8 +645,6 @@ class ArcGISSourceConfiguration(SourceConfiguration):
             from mapproxy.source import DummySource
             return DummySource(coverage=self.coverage())
 
-        # Get the supported SRS codes and formats from the configuration.
-        supported_srs = [SRS(code) for code in self.conf.get("supported_srs", [])]
         supported_formats = [file_ext(f) for f in self.conf.get("supported_formats", [])]
 
         # Construct the parameters
@@ -635,10 +664,11 @@ class ArcGISSourceConfiguration(SourceConfiguration):
         image_opts = self.image_opts(format=params.get('format'))
         return ArcGISSource(client, image_opts=image_opts, coverage=coverage,
                             res_range=res_range,
-                            supported_srs=supported_srs,
-                            supported_formats=supported_formats or None)
-
+                            supported_srs=self.supported_srs(),
+                            supported_formats=supported_formats or None,
+                            error_handler=self.on_error_handler())
 
+    @memoize
     def fi_source(self, params=None):
         from mapproxy.client.arcgis import ArcGISInfoClient
         from mapproxy.request.arcgis import create_identify_request
@@ -649,7 +679,6 @@ class ArcGISSourceConfiguration(SourceConfiguration):
         request_format = self.conf['req'].get('format')
         if request_format:
             params['format'] = request_format
-        supported_srs = [SRS(code) for code in self.conf.get('supported_srs', [])]
         fi_source = None
         if self.conf.get('opts', {}).get('featureinfo', False):
             opts = self.conf['opts']
@@ -661,7 +690,7 @@ class ArcGISSourceConfiguration(SourceConfiguration):
 
             http_client, fi_request.url = self.http_client(fi_request.url)
             fi_client = ArcGISInfoClient(fi_request,
-                supported_srs=supported_srs,
+                supported_srs=self.supported_srs(),
                 http_client=http_client,
                 tolerance=tolerance,
                 return_geometries=return_geometries,
@@ -696,7 +725,10 @@ class WMSSourceConfiguration(SourceConfiguration):
             if not has_xslt_support:
                 raise ValueError('featureinfo_xslt requires lxml. Please install.')
             fi_xslt = context.globals.abspath(fi_xslt)
-            fi_transformer = XSLTransformer(fi_xslt)
+            fi_format = conf.get('featureinfo_out_format')
+            if not fi_format:
+                fi_format = conf.get('featureinfo_format')
+            fi_transformer = XSLTransformer(fi_xslt, fi_format)
         return fi_transformer
 
     def image_opts(self, format=None):
@@ -711,7 +743,7 @@ class WMSSourceConfiguration(SourceConfiguration):
         from mapproxy.client.wms import WMSClient
         from mapproxy.request.wms import create_request
         from mapproxy.source.wms import WMSSource
-        from mapproxy.srs import SRS
+        from mapproxy.srs import SRS, SupportedSRS
 
         if not self.conf.get('wms_opts', {}).get('map', True):
             return None
@@ -728,7 +760,6 @@ class WMSSourceConfiguration(SourceConfiguration):
 
         image_opts = self.image_opts(format=params.get('format'))
 
-        supported_srs = [SRS(code) for code in self.conf.get('supported_srs', [])]
         supported_formats = [file_ext(f) for f in self.conf.get('supported_formats', [])]
         version = self.conf.get('wms_opts', {}).get('version', '1.1.1')
 
@@ -766,9 +797,10 @@ class WMSSourceConfiguration(SourceConfiguration):
         return WMSSource(client, image_opts=image_opts, coverage=coverage,
                          res_range=res_range, transparent_color=transparent_color,
                          transparent_color_tolerance=transparent_color_tolerance,
-                         supported_srs=supported_srs,
+                         supported_srs=self.supported_srs(),
                          supported_formats=supported_formats or None,
-                         fwd_req_params=fwd_req_params)
+                         fwd_req_params=fwd_req_params,
+                         error_handler=self.on_error_handler())
 
     def fi_source(self, params=None):
         from mapproxy.client.wms import WMSInfoClient
@@ -780,7 +812,6 @@ class WMSSourceConfiguration(SourceConfiguration):
         request_format = self.conf['req'].get('format')
         if request_format:
             params['format'] = request_format
-        supported_srs = [SRS(code) for code in self.conf.get('supported_srs', [])]
         fi_source = None
         if self.conf.get('wms_opts', {}).get('featureinfo', False):
             wms_opts = self.conf['wms_opts']
@@ -795,9 +826,11 @@ class WMSSourceConfiguration(SourceConfiguration):
                                                      self.context)
 
             http_client, fi_request.url = self.http_client(fi_request.url)
-            fi_client = WMSInfoClient(fi_request, supported_srs=supported_srs,
+            fi_client = WMSInfoClient(fi_request, supported_srs=self.supported_srs(),
                                       http_client=http_client)
-            fi_source = WMSInfoSource(fi_client, fi_transformer=fi_transformer)
+            coverage = self.coverage()
+            fi_source = WMSInfoSource(fi_client, fi_transformer=fi_transformer,
+                                      coverage=coverage)
         return fi_source
 
     def lg_source(self, params=None):
@@ -942,7 +975,7 @@ class TileSourceConfiguration(SourceConfiguration):
 
         grid_name = self.conf.get('grid')
         if grid_name is None:
-            log.warn("tile source for %s does not have a grid configured and defaults to GLOBAL_MERCATOR. default will change with MapProxy 2.0", url)
+            log.warning("tile source for %s does not have a grid configured and defaults to GLOBAL_MERCATOR. default will change with MapProxy 2.0", url)
             grid_name = "GLOBAL_MERCATOR"
 
         grid = self.context.grids[grid_name].tile_grid()
@@ -990,7 +1023,7 @@ class CacheConfiguration(ConfigurationBase):
         cache_dir = self.conf.get('cache', {}).get('directory')
         if cache_dir:
             if self.conf.get('cache_dir'):
-                log.warn('found cache.directory and cache_dir option for %s, ignoring cache_dir',
+                log.warning('found cache.directory and cache_dir option for %s, ignoring cache_dir',
                 self.conf['name'])
             return self.context.globals.abspath(cache_dir)
 
@@ -1028,7 +1061,7 @@ class CacheConfiguration(ConfigurationBase):
             global_key='cache.link_single_color_images')
 
         if link_single_color_images and sys.platform == 'win32':
-            log.warn('link_single_color_images not supported on windows')
+            log.warning('link_single_color_images not supported on windows')
             link_single_color_images = False
 
         return FileCache(
@@ -1109,6 +1142,15 @@ class CacheConfiguration(ConfigurationBase):
         profile_name = self.context.globals.get_value('cache.profile_name', self.conf,
             global_key='cache.s3.profile_name')
 
+        region_name = self.context.globals.get_value('cache.region_name', self.conf,
+            global_key='cache.s3.region_name')
+
+        endpoint_url = self.context.globals.get_value('cache.endpoint_url', self.conf,
+            global_key='cache.s3.endpoint_url')
+
+        access_control_list = self.context.globals.get_value('cache.access_control_list', self.conf,
+            global_key='cache.s3.access_control_list')
+
         directory_layout = self.conf['cache'].get('directory_layout', 'tms')
 
         base_path = self.conf['cache'].get('directory', None)
@@ -1121,6 +1163,9 @@ class CacheConfiguration(ConfigurationBase):
             directory_layout=directory_layout,
             bucket_name=bucket_name,
             profile_name=profile_name,
+            region_name=region_name,
+            endpoint_url=endpoint_url,
+            access_control_list=access_control_list
         )
 
     def _sqlite_cache(self, grid_conf, file_ext):
@@ -1407,8 +1452,8 @@ class CacheConfiguration(ConfigurationBase):
         from mapproxy.layer import map_extent_from_grid, merge_layer_extents
 
         base_image_opts = self.image_opts()
-        if self.conf.get('format') == 'mixed' and not self.conf.get('request_format') == 'image/png':
-            raise ConfigurationError('request_format must be set to image/png if mixed mode is enabled')
+        if self.conf.get('format') == 'mixed' and not self.conf.get('request_format') in [ 'image/png', 'image/vnd.jpeg-png' ]:
+            raise ConfigurationError('request_format must be set to image/png or image/vnd.jpeg-png if mixed mode is enabled')
         request_format = self.conf.get('request_format') or self.conf.get('format')
         if '/' in request_format:
             request_format_ext = request_format.split('/', 1)[1]
@@ -1428,6 +1473,22 @@ class CacheConfiguration(ConfigurationBase):
         concurrent_tile_creators = self.context.globals.get_value('concurrent_tile_creators', self.conf,
             global_key='cache.concurrent_tile_creators')
 
+        cache_rescaled_tiles = self.conf.get('cache_rescaled_tiles')
+        upscale_tiles = self.conf.get('upscale_tiles', 0)
+        if upscale_tiles < 0:
+            raise ConfigurationError("upscale_tiles must be positive")
+        downscale_tiles = self.conf.get('downscale_tiles', 0)
+        if downscale_tiles < 0:
+            raise ConfigurationError("downscale_tiles must be positive")
+        if upscale_tiles and downscale_tiles:
+            raise ConfigurationError("cannot use both upscale_tiles and downscale_tiles")
+
+        rescale_tiles = 0
+        if upscale_tiles:
+            rescale_tiles = -upscale_tiles
+        if downscale_tiles:
+            rescale_tiles = downscale_tiles
+
         renderd_address = self.context.globals.get_value('renderd.address', self.conf)
 
         band_merger = None
@@ -1508,6 +1569,8 @@ class CacheConfiguration(ConfigurationBase):
                 pre_store_filter=tile_filter,
                 tile_creator_class=tile_creator_class,
                 bulk_meta_tiles=bulk_meta_tiles,
+                cache_rescaled_tiles=cache_rescaled_tiles,
+                rescale_tiles=rescale_tiles,
             )
             extent = merge_layer_extents(sources)
             if extent.is_default:
@@ -1519,7 +1582,7 @@ class CacheConfiguration(ConfigurationBase):
     def grid_confs(self):
         grid_names = self.conf.get('grids')
         if grid_names is None:
-            log.warn('cache %s does not have any grids. default will change from [GLOBAL_MERCATOR] to [GLOBAL_WEBMERCATOR] with MapProxy 2.0',
+            log.warning('cache %s does not have any grids. default will change from [GLOBAL_MERCATOR] to [GLOBAL_WEBMERCATOR] with MapProxy 2.0',
                 self.conf['name'])
             grid_names = ['GLOBAL_MERCATOR']
         return [(g, self.context.grids[g]) for g in grid_names]
@@ -1538,12 +1601,13 @@ class CacheConfiguration(ConfigurationBase):
                 main_grid = grid
             caches.append((CacheMapLayer(tile_manager, extent=extent, image_opts=image_opts,
                                          max_tile_limit=max_tile_limit),
-                          (grid.srs,)))
+                          grid.srs))
 
         if len(caches) == 1:
             layer = caches[0][0]
         else:
-            layer = SRSConditional(caches, caches[0][0].extent, opacity=image_opts.opacity)
+            layer = SRSConditional(caches, caches[0][0].extent, opacity=image_opts.opacity,
+                                   preferred_srs=self.context.globals.preferred_srs)
 
         if 'use_direct_from_level' in self.conf:
             self.conf['use_direct_from_res'] = main_grid.resolution(self.conf['use_direct_from_level'])
@@ -1670,6 +1734,7 @@ class LayerConfiguration(ConfigurationBase):
         from mapproxy.cache.dummy import DummyCache
 
         sources = []
+        fi_only_sources = []
         if 'tile_sources' in self.conf:
             sources = self.conf['tile_sources']
         else:
@@ -1683,18 +1748,31 @@ class LayerConfiguration(ConfigurationBase):
                             continue
                         # and WMS layers with map: False (i.e. FeatureInfo only sources)
                         if src_conf['type'] == 'wms' and src_conf.get('wms_opts', {}).get('map', True) == False:
+                            fi_only_sources.append(source_name)
                             continue
 
                     return []
                 sources.append(source_name)
 
             if len(sources) > 1:
+                # skip layers with more then one source
                 return []
 
+
         dimensions = self.dimensions()
 
         tile_layers = []
         for cache_name in sources:
+            fi_sources = []
+            fi_source_names = cache_source_names(self.context, cache_name)
+
+            for fi_source_name in fi_source_names + fi_only_sources:
+                if fi_source_name not in self.context.sources: continue
+                if not hasattr(self.context.sources[fi_source_name], 'fi_source'): continue
+                fi_source = self.context.sources[fi_source_name].fi_source()
+                if fi_source:
+                    fi_sources.append(fi_source)
+
             for grid, extent, cache_source in self.context.caches[cache_name].caches():
                 if dimensions and not isinstance(cache_source.cache, DummyCache):
                     # caching of dimension layers is not supported yet
@@ -1715,8 +1793,15 @@ class LayerConfiguration(ConfigurationBase):
                 md['format'] = self.context.caches[cache_name].image_opts().format
                 md['cache_name'] = cache_name
                 md['extent'] = extent
-                tile_layers.append(TileLayer(self.conf['name'], self.conf['title'],
-                                             md, cache_source, dimensions=dimensions))
+                tile_layers.append(
+                    TileLayer(
+                        self.conf['name'], self.conf['title'],
+                        info_sources=fi_sources,
+                        md=md,
+                        tile_manager=cache_source,
+                        dimensions=dimensions,
+                    )
+                )
 
         return tile_layers
 
@@ -1832,19 +1917,35 @@ class ServiceConfiguration(ConfigurationBase):
         max_tile_age = self.context.globals.get_value('tiles.expires_hours')
         max_tile_age *= 60 * 60 # seconds
 
+        info_formats = conf.get('featureinfo_formats', [])
+        info_formats = odict((f['suffix'], f['mimetype']) for f in info_formats)
+
         if kvp is None and restful is None:
             kvp = restful = True
 
         services = []
         if kvp:
-            services.append(WMTSServer(layers, md, max_tile_age=max_tile_age))
+            services.append(
+                WMTSServer(
+                    layers, md, max_tile_age=max_tile_age,
+                    info_formats=info_formats,
+                )
+            )
+
         if restful:
             template = conf.get('restful_template')
+            fi_template = conf.get('restful_featureinfo_template')
             if template and '{{' in template:
                 # TODO remove warning in 1.6
-                log.warn("double braces in WMTS restful_template are deprecated {{x}} -> {x}")
-            services.append(WMTSRestServer(layers, md, template=template,
-                max_tile_age=max_tile_age))
+                log.warning("double braces in WMTS restful_template are deprecated {{x}} -> {x}")
+            services.append(
+                WMTSRestServer(
+                    layers, md, template=template,
+                    fi_template=fi_template,
+                    max_tile_age=max_tile_age,
+                    info_formats=info_formats,
+                    )
+            )
 
         return services
 
@@ -1874,7 +1975,7 @@ class ServiceConfiguration(ConfigurationBase):
         for format in image_formats_names:
             opts = self.context.globals.image_options.image_opts({}, format)
             if opts.format in image_formats:
-                log.warn('duplicate mime-type for WMS image_formats: "%s" already configured, will use last format',
+                log.warning('duplicate mime-type for WMS image_formats: "%s" already configured, will use last format',
                     opts.format)
             image_formats[opts.format] = opts
         info_types = conf.get('featureinfo_types')
@@ -1882,10 +1983,6 @@ class ServiceConfiguration(ConfigurationBase):
         self.context.globals.base_config.wms.srs = srs
         srs_extents = extents_for_srs(conf.get('bbox_srs', []))
 
-        versions = conf.get('versions')
-        if versions:
-            versions = sorted([Version(v) for v in versions])
-
         versions = conf.get('versions')
         if versions:
             versions = sorted([Version(v) for v in versions])
@@ -1975,13 +2072,13 @@ def load_configuration(mapproxy_conf, seed=False, ignore_warnings=True, renderd=
 
     errors, informal_only = validate_options(conf_dict)
     for error in errors:
-        log.warn(error)
+        log.warning(error)
     if not informal_only or (errors and not ignore_warnings):
         raise ConfigurationError('invalid configuration')
 
     errors = validate_references(conf_dict)
     for error in errors:
-        log.warn(error)
+        log.warning(error)
 
     return ProxyConfiguration(conf_dict, conf_base_dir=conf_base_dir, seed=seed,
         renderd=renderd)
@@ -2053,5 +2150,3 @@ def parse_color(color):
         return r, g, b, a
 
     return r, g, b
-
-
