@@ -1,5 +1,5 @@
 # This file is part of the MapProxy project.
-# Copyright (C) 2010 Omniscale <http://omniscale.de>
+# Copyright (C) 2010-2017 Omniscale <http://omniscale.de>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,31 +13,34 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import with_statement
 
 import os
 import time
-import sys
 
-from mapproxy.client.http import HTTPClient, HTTPClientError
-from mapproxy.client.tile import TMSClient, TileClient, TileURLTemplate
+import pytest
+
+from mapproxy.client.http import HTTPClient, HTTPClientError, supports_ssl_default_context
+from mapproxy.client.tile import TileClient, TileURLTemplate
 from mapproxy.client.wms import WMSClient, WMSInfoClient
 from mapproxy.grid import tile_grid
 from mapproxy.layer import MapQuery, InfoQuery
-from mapproxy.request.wms import WMS111MapRequest, WMS100MapRequest,\
-                                 WMS130MapRequest, WMS111FeatureInfoRequest
-from mapproxy.srs import SRS
-from mapproxy.test.unit.test_cache import MockHTTPClient
-from mapproxy.test.http import mock_httpd, query_eq, assert_query_eq, wms_query_eq
+from mapproxy.request.wms import (
+    WMS111MapRequest,
+    WMS100MapRequest,
+    WMS130MapRequest,
+    WMS111FeatureInfoRequest,
+)
+from mapproxy.source import SourceError
+from mapproxy.srs import SRS, SupportedSRS
 from mapproxy.test.helper import assert_re, TempFile
+from mapproxy.test.http import mock_httpd, query_eq, assert_query_eq, wms_query_eq
+from mapproxy.test.unit.test_cache import MockHTTPClient
 
-from nose.tools import eq_
-from nose.plugins.skip import SkipTest
-from nose.plugins.attrib import attr
 
 TESTSERVER_ADDRESS = ('127.0.0.1', 56413)
 TESTSERVER_URL = 'http://%s:%s' % TESTSERVER_ADDRESS
 
+
 class TestHTTPClient(object):
     def setup(self):
         self.client = HTTPClient()
@@ -56,6 +59,7 @@ class TestHTTPClient(object):
             assert_re(e.args[0], r'HTTP Error ".*": 500')
         else:
             assert False, 'expected HTTPClientError'
+
     def test_invalid_url_type(self):
         try:
             self.client.open('htp://example.org')
@@ -63,6 +67,7 @@ class TestHTTPClient(object):
             assert_re(e.args[0], r'No response .* "htp://example.*": unknown url type')
         else:
             assert False, 'expected HTTPClientError'
+
     def test_invalid_url(self):
         try:
             self.client.open('this is not a url')
@@ -70,6 +75,7 @@ class TestHTTPClient(object):
             assert_re(e.args[0], r'URL not correct "this is not.*": unknown url type')
         else:
             assert False, 'expected HTTPClientError'
+
     def test_unknown_host(self):
         try:
             self.client.open('http://thishostshouldnotexist000136really42.org')
@@ -77,6 +83,7 @@ class TestHTTPClient(object):
             assert_re(e.args[0], r'No response .* "http://thishost.*": .*')
         else:
             assert False, 'expected HTTPClientError'
+
     def test_no_connect(self):
         try:
             self.client.open('http://localhost:53871')
@@ -85,39 +92,35 @@ class TestHTTPClient(object):
         else:
             assert False, 'expected HTTPClientError'
 
-    @attr('online')
-    def test_https_no_ssl_module_error(self):
-        from mapproxy.client import http
-        old_ssl = http.ssl
+    def test_internal_error_hide_error_details(self):
         try:
-            http.ssl = None
-            try:
-                self.client = HTTPClient('https://www.google.com/')
-            except ImportError:
-                pass
-            else:
-                assert False, 'no ImportError for missing ssl module'
-        finally:
-            http.ssl = old_ssl
+            with mock_httpd(TESTSERVER_ADDRESS, [({'path': '/'},
+                                                  {'status': '500', 'body': b''})]):
+                HTTPClient(hide_error_details=True).open(TESTSERVER_URL + '/')
+        except HTTPClientError as e:
+            assert_re(e.args[0], r'HTTP Error \(see logs for URL and reason\).')
+        else:
+            assert False, 'expected HTTPClientError'
 
-    @attr('online')
-    def test_https_no_ssl_module_insecure(self):
-        from mapproxy.client import http
-        old_ssl = http.ssl
+    @pytest.mark.online
+    def test_https_untrusted_root(self):
+        if not supports_ssl_default_context:
+            raise pytest.skip("old python versions require ssl_ca_certs")
+        self.client = HTTPClient('https://untrusted-root.badssl.com/')
         try:
-            http.ssl = None
-            self.client = HTTPClient('https://www.google.com/', insecure=True)
-            self.client.open('https://www.google.com/')
-        finally:
-            http.ssl = old_ssl
+            self.client.open('https://untrusted-root.badssl.com/')
+        except HTTPClientError as e:
+            assert_re(e.args[0], r'Could not verify connection to URL')
 
-    @attr('online')
-    def test_https_valid_cert(self):
-        try:
-            import ssl; ssl
-        except ImportError:
-            raise SkipTest()
+    @pytest.mark.online
+    def test_https_insecure(self):
+        self.client = HTTPClient(
+            'https://untrusted-root.badssl.com/', insecure=True)
+        self.client.open('https://untrusted-root.badssl.com/')
 
+    @pytest.mark.online
+    def test_https_valid_ca_cert_file(self):
+        # verify with fixed ca_certs file
         cert_file = '/etc/ssl/certs/ca-certificates.crt'
         if os.path.exists(cert_file):
             self.client = HTTPClient('https://www.google.com/', ssl_ca_certs=cert_file)
@@ -129,17 +132,24 @@ class TestHTTPClient(object):
                 self.client = HTTPClient('https://www.google.com/', ssl_ca_certs=tmp)
                 self.client.open('https://www.google.com/')
 
-    @attr('online')
-    def test_https_invalid_cert(self):
-        try:
-            import ssl; ssl
-        except ImportError:
-            raise SkipTest()
+    @pytest.mark.online
+    def test_https_valid_default_cert(self):
+        # modern python should verify by default
+        if not supports_ssl_default_context:
+            raise pytest.skip("old python versions require ssl_ca_certs")
+        self.client = HTTPClient('https://www.google.com/')
+        self.client.open('https://www.google.com/')
 
+    @pytest.mark.online
+    def test_https_invalid_cert(self):
+        # load 'wrong' root cert
         with TempFile() as tmp:
-            self.client = HTTPClient('https://www.google.com/', ssl_ca_certs=tmp)
+            with open(tmp, 'wb') as f:
+                f.write(GOOGLE_ROOT_CERT)
+            self.client = HTTPClient(
+                'https://untrusted-root.badssl.com/', ssl_ca_certs=tmp)
             try:
-                self.client.open('https://www.google.com/')
+                self.client.open('https://untrusted-root.badssl.com/')
             except HTTPClientError as e:
                 assert_re(e.args[0], r'Could not verify connection to URL')
 
@@ -149,15 +159,12 @@ class TestHTTPClient(object):
 
         import mapproxy.client.http
 
-        old_timeout = mapproxy.client.http._max_set_timeout
-        mapproxy.client.http._max_set_timeout = None
-
         client1 = HTTPClient(timeout=0.1)
         client2 = HTTPClient(timeout=0.5)
         with mock_httpd(TESTSERVER_ADDRESS, [test_req]):
             try:
                 start = time.time()
-                client1.open(TESTSERVER_URL+'/')
+                client1.open(TESTSERVER_URL + '/')
             except HTTPClientError as ex:
                 assert 'timed out' in ex.args[0]
             else:
@@ -167,7 +174,7 @@ class TestHTTPClient(object):
         with mock_httpd(TESTSERVER_ADDRESS, [test_req]):
             try:
                 start = time.time()
-                client2.open(TESTSERVER_URL+'/')
+                client2.open(TESTSERVER_URL + '/')
             except HTTPClientError as ex:
                 assert 'timed out' in ex.args[0]
             else:
@@ -178,12 +185,53 @@ class TestHTTPClient(object):
         assert 0.1 <= duration1 < 0.5, duration1
         assert 0.5 <= duration2 < 0.9, duration2
 
-        mapproxy.client.http._max_set_timeout = old_timeout
 
-# Equifax Secure Certificate Authority
-# Expires: 2018-08-22
+# root certificates for google.com, if no ca-certificates.cert
+# file is found
 GOOGLE_ROOT_CERT = b"""
 -----BEGIN CERTIFICATE-----
+MIIDVDCCAjygAwIBAgIDAjRWMA0GCSqGSIb3DQEBBQUAMEIxCzAJBgNVBAYTAlVT
+MRYwFAYDVQQKEw1HZW9UcnVzdCBJbmMuMRswGQYDVQQDExJHZW9UcnVzdCBHbG9i
+YWwgQ0EwHhcNMDIwNTIxMDQwMDAwWhcNMjIwNTIxMDQwMDAwWjBCMQswCQYDVQQG
+EwJVUzEWMBQGA1UEChMNR2VvVHJ1c3QgSW5jLjEbMBkGA1UEAxMSR2VvVHJ1c3Qg
+R2xvYmFsIENBMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA2swYYzD9
+9BcjGlZ+W988bDjkcbd4kdS8odhM+KhDtgPpTSEHCIjaWC9mOSm9BXiLnTjoBbdq
+fnGk5sRgprDvgOSJKA+eJdbtg/OtppHHmMlCGDUUna2YRpIuT8rxh0PBFpVXLVDv
+iS2Aelet8u5fa9IAjbkU+BQVNdnARqN7csiRv8lVK83Qlz6cJmTM386DGXHKTubU
+1XupGc1V3sjs0l44U+VcT4wt/lAjNvxm5suOpDkZALeVAjmRCw7+OC7RHQWa9k0+
+bw8HHa8sHo9gOeL6NlMTOdReJivbPagUvTLrGAMoUgRx5aszPeE4uwc2hGKceeoW
+MPRfwCvocWvk+QIDAQABo1MwUTAPBgNVHRMBAf8EBTADAQH/MB0GA1UdDgQWBBTA
+ephojYn7qwVkDBF9qn1luMrMTjAfBgNVHSMEGDAWgBTAephojYn7qwVkDBF9qn1l
+uMrMTjANBgkqhkiG9w0BAQUFAAOCAQEANeMpauUvXVSOKVCUn5kaFOSPeCpilKIn
+Z57QzxpeR+nBsqTP3UEaBU6bS+5Kb1VSsyShNwrrZHYqLizz/Tt1kL/6cdjHPTfS
+tQWVYrmm3ok9Nns4d0iXrKYgjy6myQzCsplFAMfOEVEiIuCl6rYVSAlk6l5PdPcF
+PseKUgzbFbS9bZvlxrFUaKnjaZC2mqUPuLk/IH2uSrW4nOQdtqvmlKXBx4Ot2/Un
+hw4EbNX/3aBd7YdStysVAq45pmp06drE57xNNB6pXE0zX5IJL4hmXXeXxx12E6nV
+5fEWCRE11azbJHFwLJhWC9kXtNHjUStedejV0NxPNO3CBWaAocvmMw==
+-----END CERTIFICATE-----
+-----BEGIN CERTIFICATE-----
+MIIDujCCAqKgAwIBAgILBAAAAAABD4Ym5g0wDQYJKoZIhvcNAQEFBQAwTDEgMB4G
+A1UECxMXR2xvYmFsU2lnbiBSb290IENBIC0gUjIxEzARBgNVBAoTCkdsb2JhbFNp
+Z24xEzARBgNVBAMTCkdsb2JhbFNpZ24wHhcNMDYxMjE1MDgwMDAwWhcNMjExMjE1
+MDgwMDAwWjBMMSAwHgYDVQQLExdHbG9iYWxTaWduIFJvb3QgQ0EgLSBSMjETMBEG
+A1UEChMKR2xvYmFsU2lnbjETMBEGA1UEAxMKR2xvYmFsU2lnbjCCASIwDQYJKoZI
+hvcNAQEBBQADggEPADCCAQoCggEBAKbPJA6+Lm8omUVCxKs+IVSbC9N/hHD6ErPL
+v4dfxn+G07IwXNb9rfF73OX4YJYJkhD10FPe+3t+c4isUoh7SqbKSaZeqKeMWhG8
+eoLrvozps6yWJQeXSpkqBy+0Hne/ig+1AnwblrjFuTosvNYSuetZfeLQBoZfXklq
+tTleiDTsvHgMCJiEbKjNS7SgfQx5TfC4LcshytVsW33hoCmEofnTlEnLJGKRILzd
+C9XZzPnqJworc5HGnRusyMvo4KD0L5CLTfuwNhv2GXqF4G3yYROIXJ/gkwpRl4pa
+zq+r1feqCapgvdzZX99yqWATXgAByUr6P6TqBwMhAo6CygPCm48CAwEAAaOBnDCB
+mTAOBgNVHQ8BAf8EBAMCAQYwDwYDVR0TAQH/BAUwAwEB/zAdBgNVHQ4EFgQUm+IH
+V2ccHsBqBt5ZtJot39wZhi4wNgYDVR0fBC8wLTAroCmgJ4YlaHR0cDovL2NybC5n
+bG9iYWxzaWduLm5ldC9yb290LXIyLmNybDAfBgNVHSMEGDAWgBSb4gdXZxwewGoG
+3lm0mi3f3BmGLjANBgkqhkiG9w0BAQUFAAOCAQEAmYFThxxol4aR7OBKuEQLq4Gs
+J0/WwbgcQ3izDJr86iw8bmEbTUsp9Z8FHSbBuOmDAGJFtqkIk7mpM0sYmsL4h4hO
+291xNBrBVNpGP+DTKqttVCL1OmLNIG+6KYnX3ZHu01yiPqFbQfXf5WRDLenVOavS
+ot+3i9DAgBkcRcAtjOj4LaR0VknFBbVPFd5uRHg5h6h+u/N5GJG79G+dwfCMNYxd
+AfvDbbnvRG15RjF+Cv6pgsH/76tuIMRQyV+dTZsXjAzlAcmgQWpzU/qlULRuJQ/7
+TBj0/VLZjmmx6BEP3ojY+x1J96relc8geMJgEtslQIxq/H5COEBkEveegeGTLg==
+-----END CERTIFICATE-----
+-----BEGIN CERTIFICATE-----
 MIIDIDCCAomgAwIBAgIENd70zzANBgkqhkiG9w0BAQUFADBOMQswCQYDVQQGEwJV
 UzEQMA4GA1UEChMHRXF1aWZheDEtMCsGA1UECxMkRXF1aWZheCBTZWN1cmUgQ2Vy
 dGlmaWNhdGUgQXV0aG9yaXR5MB4XDTk4MDgyMjE2NDE1MVoXDTE4MDgyMjE2NDE1
@@ -204,16 +252,6 @@ A4GBAFjOKer89961zgK5F7WF0bnj4JXMJTENAKaSbn+2kmOeUJXRmm/kEd5jhW6Y
 -----END CERTIFICATE-----
 """
 
-
-class TestTMSClient(object):
-    def setup(self):
-        self.client = TMSClient(TESTSERVER_URL)
-    def test_get_tile(self):
-        with mock_httpd(TESTSERVER_ADDRESS, [({'path': '/9/5/13.png'},
-                                                {'body': b'tile', 'headers': {'content-type': 'image/png'}})]):
-            resp = self.client.get_tile((5, 13, 9)).source.read()
-            eq_(resp, b'tile')
-
 class TestTileClient(object):
     def test_tc_path(self):
         template = TileURLTemplate(TESTSERVER_URL + '/%(tc_path)s.png')
@@ -222,7 +260,7 @@ class TestTileClient(object):
                                               {'body': b'tile',
                                                'headers': {'content-type': 'image/png'}})]):
             resp = client.get_tile((5, 13, 9)).source.read()
-            eq_(resp, b'tile')
+            assert resp == b'tile'
 
     def test_quadkey(self):
         template = TileURLTemplate(TESTSERVER_URL + '/key=%(quadkey)s&format=%(format)s')
@@ -231,7 +269,7 @@ class TestTileClient(object):
                                               {'body': b'tile',
                                                'headers': {'content-type': 'image/png'}})]):
             resp = client.get_tile((5, 13, 9)).source.read()
-            eq_(resp, b'tile')
+            assert resp == b'tile'
     def test_xyz(self):
         template = TileURLTemplate(TESTSERVER_URL + '/x=%(x)s&y=%(y)s&z=%(z)s&format=%(format)s')
         client = TileClient(template)
@@ -239,7 +277,7 @@ class TestTileClient(object):
                                               {'body': b'tile',
                                                'headers': {'content-type': 'image/png'}})]):
             resp = client.get_tile((5, 13, 9)).source.read()
-            eq_(resp, b'tile')
+            assert resp == b'tile'
 
     def test_arcgiscache_path(self):
         template = TileURLTemplate(TESTSERVER_URL + '/%(arcgiscache_path)s.png')
@@ -248,7 +286,7 @@ class TestTileClient(object):
                                               {'body': b'tile',
                                                'headers': {'content-type': 'image/png'}})]):
             resp = client.get_tile((5, 13, 9)).source.read()
-            eq_(resp, b'tile')
+            assert resp == b'tile'
 
     def test_bbox(self):
         grid = tile_grid(4326)
@@ -258,7 +296,27 @@ class TestTileClient(object):
                                               {'body': b'tile',
                                                'headers': {'content-type': 'image/png'}})]):
             resp = client.get_tile((0, 1, 2)).source.read()
-            eq_(resp, b'tile')
+            assert resp == b'tile'
+
+
+class TestWMSClient(object):
+    def test_no_image(self, caplog):
+        try:
+            with mock_httpd(TESTSERVER_ADDRESS, [({'path': '/service?map=foo&layers=foo&transparent=true&bbox=-200000,-200000,200000,200000&width=512&height=512&srs=EPSG%3A900913&format=image%2Fpng&request=GetMap&version=1.1.1&service=WMS&styles='},
+                                                  {'status': '200', 'body': b'x' * 1000,
+                                                   'headers': {'content-type': 'application/foo'},
+                                                  })]):
+                req = WMS111MapRequest(url=TESTSERVER_URL + '/service?map=foo',
+                                        param={'layers':'foo', 'transparent': 'true'})
+                query = MapQuery((-200000, -200000, 200000, 200000), (512, 512), SRS(900913), 'png')
+                wms = WMSClient(req).retrieve(query, 'png')
+        except SourceError:
+            assert len(caplog.record_tuples) == 1
+            assert ("'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx' [output truncated]"
+                in caplog.record_tuples[0][2])
+        else:
+            assert False, 'expected no image returned error'
+
 
 class TestCombinedWMSClient(object):
     def setup(self):
@@ -274,8 +332,8 @@ class TestCombinedWMSClient(object):
         req = MapQuery((-200000, -200000, 200000, 200000), (512, 512), SRS(900913), 'png')
 
         combined = wms1.combined_client(wms2, req)
-        eq_(combined.request_template.params.layers, ['foo', 'bar'])
-        eq_(combined.request_template.url, TESTSERVER_URL + '/service?map=foo')
+        assert combined.request_template.params.layers == ['foo', 'bar']
+        assert combined.request_template.url == TESTSERVER_URL + '/service?map=foo'
 
     def test_combine_different_url(self):
         req1 = WMS111MapRequest(url=TESTSERVER_URL + '/service?map=bar',
@@ -294,7 +352,7 @@ class TestWMSInfoClient(object):
     def test_transform_fi_request_supported_srs(self):
         req = WMS111FeatureInfoRequest(url=TESTSERVER_URL + '/service?map=foo', param={'layers':'foo'})
         http = MockHTTPClient()
-        wms = WMSInfoClient(req, http_client=http, supported_srs=[SRS(25832)])
+        wms = WMSInfoClient(req, http_client=http, supported_srs=SupportedSRS([SRS(25832)]))
         fi_req = InfoQuery((8, 50, 9, 51), (512, 512),
                            SRS(4326), (128, 64), 'text/plain')
 
@@ -328,12 +386,12 @@ class TestWMSMapRequest100(object):
         self.r = WMS100MapRequest(param=dict(layers='foo', version='1.1.1', request='GetMap'))
         self.r.params = self.r.adapt_params_to_version()
     def test_version(self):
-        eq_(self.r.params['WMTVER'], '1.0.0')
+        assert self.r.params['WMTVER'] == '1.0.0'
         assert 'VERSION' not in self.r.params
     def test_service(self):
         assert 'SERVICE' not in self.r.params
     def test_request(self):
-        eq_(self.r.params['request'], 'map')
+        assert self.r.params['request'] == 'map'
     def test_str(self):
         assert_query_eq(str(self.r.params), 'layers=foo&styles=&request=map&wmtver=1.0.0')
 
@@ -342,12 +400,12 @@ class TestWMSMapRequest130(object):
         self.r = WMS130MapRequest(param=dict(layers='foo', WMTVER='1.0.0'))
         self.r.params = self.r.adapt_params_to_version()
     def test_version(self):
-        eq_(self.r.params['version'], '1.3.0')
+        assert self.r.params['version'] == '1.3.0'
         assert 'WMTVER' not in self.r.params
     def test_service(self):
-        eq_(self.r.params['service'], 'WMS' )
+        assert self.r.params['service'] == 'WMS'
     def test_request(self):
-        eq_(self.r.params['request'], 'GetMap')
+        assert self.r.params['request'] == 'GetMap'
     def test_str(self):
         query_eq(str(self.r.params), 'layers=foo&styles=&service=WMS&request=GetMap&version=1.3.0')
 
@@ -356,5 +414,5 @@ class TestWMSMapRequest111(object):
         self.r = WMS111MapRequest(param=dict(layers='foo', WMTVER='1.0.0'))
         self.r.params = self.r.adapt_params_to_version()
     def test_version(self):
-        eq_(self.r.params['version'], '1.1.1')
+        assert self.r.params['version'] == '1.1.1'
         assert 'WMTVER' not in self.r.params
